local QTC = LibStub("LibQTipClick-1.1")
local LDB = LibStub("LibDataBroker-1.1")
BES = LibStub("AceAddon-3.0"):NewAddon("Broker_BlizzardEquipmentSet", "AceConsole-3.0", "AceTimer-3.0", "AceEvent-3.0")

--------------------------------------------------------------------------------
-- Tooltip functions
local function Tooltip_ClickCell(event, cell, arg, button)
	UseEquipmentSet(arg)
end

local function Tooltip_Show(frame)
	if QTC:IsAcquired("Broker_BlizzEquipmentSet") then QTC:Release(BES.tooltip) end
	BES.tooltip = QTC:Acquire("Broker_BlizzEquipmentSet", 2, "LEFT", "LEFT")
	BES.tooltip:SetAutoHideDelay(1, frame)
	-- Assign the callback to the tooltip
	BES.tooltip:SetCallback("OnMouseDown", Tooltip_ClickCell)
	
	for i = 1, GetNumEquipmentSets() do
		name, icon = GetEquipmentSetInfo(i)
		local y, x = BES.tooltip:AddLine()
		y, x = BES.tooltip:SetCell(y, 1, "|T" .. icon ..":16|t", name)
		y, x = BES.tooltip:SetCell(y, 2, name, name)
	end
	BES.tooltip:SmartAnchorTo(frame)
	BES.tooltip:Show()
end

--[[local function Tooltip_Hide()
	-- Release the tooltip
	BES.tooltip:Hide()
	QTC:Release(BES.tooltip)
end]]--

--------------------------------------------------------------------------------
-- Ace addon events
function BES:OnInitialize()
    -- Called when the addon is loaded
    self.lDBObject = {
		type = "data source",
		label = "BlizzEquipmentSet",
		icon = "Interface\\Icons\\INV_Misc_QuestionMark",
		OnEnter = Tooltip_Show,
	}
	
	LDB:NewDataObject("BlizzEquipmentSet", self.lDBObject)
end

function BES:OnEnable()
    -- Called when the addon is enabled
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateLDBObject")
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", "OnInventoryChanged")
end

function BES:OnDisable()
    -- Called when the addon is disabled
end

function BES:OnInventoryChanged(event, unit)
	if unit=="player" then
		self:UpdateLDBObject()
	end
end

local function matchesToEquipped(setName)
	local locations = GetEquipmentSetLocations(setName)
	local matches = true
	local equippedLocations = 0
	
	for i, location in pairs(locations)	do
		local wearing, bank, bags, slot, bag = EquipmentManager_UnpackLocation(location)
		local itemLink
		if not bags then
        	itemLink = GetInventoryItemLink( "player", slot )  --being worn
        else
            itemLink = GetContainerItemLink( bag, slot )  -- bags or bank
        end
		
		if bags and itemLink ~= nil then
			matches = false
		end
		if itemLink ~= nil then
			equippedLocations = equippedLocations + 1
		end
	end
	return matches, equippedLocations
end

function BES:UpdateLDBObject()
	local maxCoveredSlotNumber = -1
	local found = false
	for setIndex = 1, GetNumEquipmentSets() do
		local setName, setIcon = GetEquipmentSetInfo(setIndex)
		local matches, coveredSlotNumber = matchesToEquipped(setName)
		if matches and coveredSlotNumber > maxCoveredSlotNumber then
			self.lDBObject.text = setName
			self.lDBObject.icon = setIcon
			maxCoveredSlotNumber = coveredSlotNumber
			found = true
		end
	end
	
	if not found then
		self.lDBObject.text = "Custom set"
		self.lDBObject.icon = "Interface\\Icons\\INV_Misc_QuestionMark"
	end
end
